package com.example.gajz.transferrelationship.hugegraphclient;
import com.baidu.hugegraph.driver.GraphManager;
import com.baidu.hugegraph.driver.HugeClient;
import com.baidu.hugegraph.driver.SchemaManager;
import com.baidu.hugegraph.structure.constant.T;
import com.baidu.hugegraph.structure.graph.Vertex;
import com.example.gajz.transferrelationship.bean.vo.TransferredPersonVo;
import com.example.gajz.transferrelationship.thread.*;
import org.springframework.stereotype.Service;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;


/**
 *  HugeGraph
 * @author lenovo
 */
@Service
public class HugeCollectionProperties {

    /**
     * 创建转账人 集合顶点  转账人集合关系方法
     */
    public  void createCollectionProperties(List<TransferredPersonVo> transferredPersonVos){
        HugeClient hugeClient = HugeClient.builder("http://36.148.1.253:8289","hugegraph2")
                /*** 默认未开启用户权限*/
                .configUser("**", "**")
                .build();
                try {
                    //创建属性
                    SchemaManager schema = hugeClient.schema();
                    schema.propertyKey("卡号").asText().ifNotExist().create();
                    schema.propertyKey("转账次数").asText().ifNotExist().create();
                    //创建集合顶点
                    /**设置顶点名称*/
                    schema.vertexLabel("单独转账交易人数")
                            /**设置主键为传入*/
                            .useCustomizeNumberId()
                            /**设置顶点属性*/
                            .properties("转账次数")
                            /**判断设置顶点 如果已经存在就不创建*/
                            .ifNotExist().create();
                    //创建转账人顶点
                    /**设置顶点名称*/
                    schema.vertexLabel("转账人")
                            /**设置主键为传入*/
                            .useCustomizeNumberId()
                            /**判断设置顶点 如果已经存在就不创建*/
                            .ifNotExist().create();
                    //创建边
                    /**设置边名称*/
                    schema.edgeLabel("转账")
                            /**起点顶点*/
                            .sourceLabel("转账人")
                            /**终点顶点*/
                            .targetLabel("单独转账交易人数")
                            /**边参数*/
                            .properties("转账次数")
                            .ifNotExist()
                            .create();
                    GraphManager graph = hugeClient.graph();
                    for (int i=0;i<transferredPersonVos.size();i++){
                       /**添加集合顶点数据*/
                        Vertex aggregate = graph.addVertex(
                                /**设置ID*/
                                T.id, transferredPersonVos.get(i).getId(),
                                T.label, "单独转账交易人数",
                                /**设置转账次数*/
                                "转账次数", transferredPersonVos.get(i).getNumber());
                        /**添加转账人顶点数据*/
                        Vertex transferor = graph.addVertex(
                                /**设置ID*/
                                T.id, transferredPersonVos.get(i).getBankCardNo(),
                                T.label, "转账人");
                        //创建转账关系
                        transferor.addEdge("转账", aggregate,  "转账次数", transferredPersonVos.get(i).getNumber());
                    }
                    hugeClient.close();
                }catch (Exception e){
                    hugeClient.close();
                }
    }

    /**
     * 创建多条收款人顶点方法
     */
    public void  createMultiplePayees(List<TransferredPersonVo> multiplePayeesList){

        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        System.out.println("收款人导入开始"+df.format(new Date()));
        HugeClient hugeClient = HugeClient.builder("http://36.148.1.253:8289","hugegraph2")
                /*** 默认未开启用户权限*/
                .configUser("**", "**")
                .build();
                try {
                    //创建属性
                    SchemaManager schema = hugeClient.schema();
                    schema.propertyKey("收款次数").asText().ifNotExist().create();
                    //创建集合顶点
                    /**设置顶点名称*/
                    schema.vertexLabel("收款人")
                            /**设置主键为传入*/
                            .useCustomizeStringId()
                            /**设置顶点属性*/
                            .properties("收款次数")
                            /**判断设置顶点 如果已经存在就不创建*/
                            .ifNotExist().create();
                    hugeClient.close();
                    //每个线程执行多少条
                    int numberExecution=(multiplePayeesList.size()/12)+1;
                    //导入线程1
                    ThreadsMultipleVerticesOne transferAccountsOne=new ThreadsMultipleVerticesOne();
                    transferAccountsOne.ThreadsMultipleVerticesOneList(multiplePayeesList.subList(0,numberExecution));
                    Thread ThreadOne = new Thread(transferAccountsOne);
                    ThreadOne.start();
                    //导入线程2
                    ThreadsMultipleVerticesTwo transferAccountsTwo=new ThreadsMultipleVerticesTwo();
                    transferAccountsTwo.ThreadsMultipleVerticesTwoList(multiplePayeesList.subList(numberExecution,numberExecution*2));
                    Thread ThreadTwo = new Thread(transferAccountsTwo);
                    ThreadTwo.start();
                    //导入线程3
                    ThreadsMultipleVerticesThree transferAccountsThree=new ThreadsMultipleVerticesThree();
                    transferAccountsThree.ThreadsMultipleVerticesThreeList(multiplePayeesList.subList(numberExecution*2,numberExecution*3));
                    Thread ThreadThree = new Thread(transferAccountsThree);
                    ThreadThree.start();
                    //导入线程4
                    ThreadsMultipleVerticesFour transferAccountsFour=new ThreadsMultipleVerticesFour();
                    transferAccountsFour.ThreadsMultipleVerticesFourList(multiplePayeesList.subList(numberExecution*3,numberExecution*4));
                    Thread ThreadFour = new Thread(transferAccountsFour);
                    ThreadFour.start();
                    //导入线程5
                    ThreadsMultipleVerticesFive transferAccountsFive=new ThreadsMultipleVerticesFive();
                    transferAccountsFive.ThreadsMultipleVerticesFiveList(multiplePayeesList.subList(numberExecution*4,numberExecution*5));
                    Thread ThreadFive = new Thread(transferAccountsFive);
                    ThreadFive.start();
                    //导入线程6
                    ThreadsMultipleVerticesSix transferAccountsSix=new ThreadsMultipleVerticesSix();
                    transferAccountsSix.ThreadsMultipleVerticesSixList(multiplePayeesList.subList(numberExecution*5,numberExecution*6));
                    Thread ThreadSix = new Thread(transferAccountsSix);
                    ThreadSix.start();
                    //导入线程7
                    ThreadsMultipleVerticesSeven transferAccountsSeven=new ThreadsMultipleVerticesSeven();
                    transferAccountsSeven.ThreadsMultipleVerticesSevenList(multiplePayeesList.subList(numberExecution*6,numberExecution*7));
                    Thread ThreadSeven = new Thread(transferAccountsSeven);
                    ThreadSeven.start();
                    //导入线程8
                    ThreadsMultipleVerticesEight transferAccountsEight=new ThreadsMultipleVerticesEight();
                    transferAccountsEight.ThreadsMultipleVerticesEightList(multiplePayeesList.subList(numberExecution*7,numberExecution*8));
                    Thread ThreadEight = new Thread(transferAccountsEight);
                    ThreadEight.start();
                    //导入线程9
                    ThreadsMultipleVerticesNine transferAccountsNine=new ThreadsMultipleVerticesNine();
                    transferAccountsNine.ThreadsMultipleVerticesNineList(multiplePayeesList.subList(numberExecution*8,numberExecution*9));
                    Thread ThreadNine = new Thread(transferAccountsNine);
                    ThreadNine.start();
                    //导入线程10
                    ThreadsMultipleVerticesTen transferAccountsTen=new ThreadsMultipleVerticesTen();
                    transferAccountsTen.ThreadsMultipleVerticesTenList(multiplePayeesList.subList(numberExecution*9,numberExecution*10));
                    Thread ThreadTen = new Thread(transferAccountsTen);
                    ThreadTen.start();
                    //线程11
                    ThreadsMultipleVerticesEleven threadsMultipleVerticesEleven=new ThreadsMultipleVerticesEleven();
                    threadsMultipleVerticesEleven.ThreadsMultipleVerticesElevenList(multiplePayeesList.subList(numberExecution*10,numberExecution*11));
                    Thread ThreadEleven = new Thread(threadsMultipleVerticesEleven);
                    ThreadEleven.start();
                    //线程12
                    ThreadsMultipleVerticesTwelve threadsMultipleVerticesTwelve=new ThreadsMultipleVerticesTwelve();
                    threadsMultipleVerticesTwelve.ThreadsMultipleVerticesTwelveList(multiplePayeesList.subList(numberExecution*11,multiplePayeesList.size()));
                    Thread ThreadTwelve = new Thread(threadsMultipleVerticesTwelve);
                    ThreadTwelve.start();
                    //等待线程
                    ThreadOne.join();
                    ThreadTwo.join();
                    ThreadThree.join();
                    ThreadFour.join();
                    ThreadFive.join();
                    ThreadSix.join();
                    ThreadSeven.join();
                    ThreadEight.join();
                    ThreadNine.join();
                    ThreadTen.join();
                    ThreadEleven.join();
                    ThreadTwelve.join();
                    System.out.println("收款人导入结束"+df.format(new Date()));
                }catch (Exception e){
                    hugeClient.close();
                }
         }


    /**
     * 创建 转账人与收款人边
     * @param transferCollectionList
     */
    public void  transferCollectionRelationship(List<TransferredPersonVo> transferCollectionList){
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        System.out.println("转账关系导入开始"+df.format(new Date()));
        HugeClient hugeClient = HugeClient.builder("http://36.148.1.253:8289","hugegraph2")
                /*** 默认未开启用户权限*/
                .configUser("**", "**")
                .build();
                try {
                    //创建属性
                    SchemaManager schema = hugeClient.schema();
                    //创建边
                    /**设置边名称*/
                    schema.edgeLabel("多人转账")
                            /**起点顶点*/
                            .sourceLabel("转账人")
                            /**终点顶点*/
                            .targetLabel("收款人")
                            /**边参数*/
                            .properties("转账次数")
                            .ifNotExist()
                            .create();
                    hugeClient.close();
//                    GraphManager graph = hugeClient.graph();
                    //每个线程执行多少条
                    int numberExecution=transferCollectionList.size()/20;
                    //线程1
                    TransferAccountsOne transferAccountsOne=new TransferAccountsOne();
                    transferAccountsOne.TransferAccountsOneList(transferCollectionList.subList(0,numberExecution));
                    Thread ThreadOne = new Thread(transferAccountsOne);
                    ThreadOne.start();
                    //线程2
                    TransferAccountsTwo transferAccountsTwo=new TransferAccountsTwo();
                    transferAccountsTwo.TransferAccountsTwoList(transferCollectionList.subList(numberExecution,numberExecution*2));
                    Thread ThreadTwo = new Thread(transferAccountsTwo);
                    ThreadTwo.start();
                    //线程3
                    TransferAccountsThree transferAccountsThree=new TransferAccountsThree();
                    transferAccountsThree.TransferAccountsThreeList(transferCollectionList.subList(numberExecution*2,numberExecution*3));
                    Thread ThreadThree = new Thread(transferAccountsThree);
                    ThreadThree.start();
                    //线程4
                    TransferAccountsFour transferAccountsFour=new TransferAccountsFour();
                    transferAccountsFour.TransferAccountsFourList(transferCollectionList.subList(numberExecution*3,numberExecution*4));
                    Thread ThreadFour = new Thread(transferAccountsFour);
                    ThreadFour.start();
                    //线程5
                    TransferAccountsFive transferAccountsFive=new TransferAccountsFive();
                    transferAccountsFive.TransferAccountsFiveList(transferCollectionList.subList(numberExecution*4,numberExecution*5));
                    Thread ThreadFive = new Thread(transferAccountsFive);
                    ThreadFive.start();
                    //线程6
                    TransferAccountsSix transferAccountsSix=new TransferAccountsSix();
                    transferAccountsSix.TransferAccountsSixList(transferCollectionList.subList(numberExecution*5,numberExecution*6));
                    Thread ThreadSix = new Thread(transferAccountsSix);
                    ThreadSix.start();
                    //线程7
                    TransferAccountsSeven transferAccountsSeven=new TransferAccountsSeven();
                    transferAccountsSeven.TransferAccountsSevenList(transferCollectionList.subList(numberExecution*6,numberExecution*7));
                    Thread ThreadSeven = new Thread(transferAccountsSeven);
                    ThreadSeven.start();
                    //线程8
                    TransferAccountsEight transferAccountsEight=new TransferAccountsEight();
                    transferAccountsEight.TransferAccountsEightList(transferCollectionList.subList(numberExecution*7,numberExecution*8));
                    Thread ThreadEight = new Thread(transferAccountsEight);
                    ThreadEight.start();
                    //线程9
                    TransferAccountsNine transferAccountsNine=new TransferAccountsNine();
                    transferAccountsNine.TransferAccountsNineList(transferCollectionList.subList(numberExecution*8,numberExecution*9));
                    Thread ThreadNine = new Thread(transferAccountsNine);
                    ThreadNine.start();
                    //线程10
                    TransferAccountsTen transferAccountsTen=new TransferAccountsTen();
                    transferAccountsTen.TransferAccountsTenList(transferCollectionList.subList(numberExecution*9,numberExecution*10));
                    Thread ThreadTen = new Thread(transferAccountsTen);
                    ThreadTen.start();
                    //线程11
                    TransferAccountsEleven transferAccountsEleven=new TransferAccountsEleven();
                    transferAccountsEleven.TransferAccountsElevenList(transferCollectionList.subList(numberExecution*10,numberExecution*11));
                    Thread ThreadEleven = new Thread(transferAccountsEleven);
                    ThreadEleven.start();
                    //线程12
                    TransferAccountsTwelve transferAccountsTwelve=new TransferAccountsTwelve();
                    transferAccountsTwelve.TransferAccountsTwelveList(transferCollectionList.subList(numberExecution*11,numberExecution*12));
                    Thread ThreadTwelve = new Thread(transferAccountsTwelve);
                    ThreadTwelve.start();
                    //线程13
                    TransferAccountsThirteen transferAccountsThirteen=new TransferAccountsThirteen();
                    transferAccountsThirteen.TransferAccountsThirteenList(transferCollectionList.subList(numberExecution*12,numberExecution*13));
                    Thread ThreadThirteen = new Thread(transferAccountsThirteen);
                    ThreadThirteen.start();
                    //线程14
                    TransferAccountsFourteen transferAccountsFourteen=new TransferAccountsFourteen();
                    transferAccountsFourteen.TransferAccountsFourteenList(transferCollectionList.subList(numberExecution*13,numberExecution*14));
                    Thread ThreadFourteen = new Thread(transferAccountsFourteen);
                    ThreadFourteen.start();
                    //线程15
                    TransferAccountsFifteen transferAccountsFifteen=new TransferAccountsFifteen();
                    transferAccountsFifteen.TransferAccountsFifteenList(transferCollectionList.subList(numberExecution*14,numberExecution*15));
                    Thread ThreadFifteen = new Thread(transferAccountsFifteen);
                    ThreadFifteen.start();
                    //线程16
                    TransferAccountsSixteen transferAccountsSixteen=new TransferAccountsSixteen();
                    transferAccountsSixteen.TransferAccountsSixteenList(transferCollectionList.subList(numberExecution*15,numberExecution*16));
                    Thread ThreadSixteen = new Thread(transferAccountsSixteen);
                    ThreadSixteen.start();
                    //线程17
                    TransferAccountsSeventeen transferAccountsSeventeen=new TransferAccountsSeventeen();
                    transferAccountsSeventeen.TransferAccountsSeventeenList(transferCollectionList.subList(numberExecution*16,numberExecution*17));
                    Thread ThreadSeventeen = new Thread(transferAccountsSeventeen);
                    ThreadSeventeen.start();
                    //线程18
                    TransferAccountsEighteen transferAccountsEighteen=new TransferAccountsEighteen();
                    transferAccountsEighteen.TransferAccountsEighteenList(transferCollectionList.subList(numberExecution*17,numberExecution*18));
                    Thread ThreadEighteen = new Thread(transferAccountsEighteen);
                    ThreadEighteen.start();
                    //线程19
                    TransferAccountsNineteen transferAccountsNineteen=new TransferAccountsNineteen();
                    transferAccountsNineteen.TransferAccountsNineteenList(transferCollectionList.subList(numberExecution*18,numberExecution*19));
                    Thread ThreadNineteen = new Thread(transferAccountsNineteen);
                    ThreadNineteen.start();
                    //线程20
                    TransferAccountsTwenty transferAccountsTwenty=new TransferAccountsTwenty();
                    transferAccountsTwenty.TransferAccountsTwentyList(transferCollectionList.subList(numberExecution*19,transferCollectionList.size()));
                    Thread ThreadTwenty = new Thread(transferAccountsTwenty);
                    ThreadTwenty.start();
                    //等待线程
                    ThreadOne.join();
                    ThreadTwo.join();
                    ThreadThree.join();
                    ThreadFour.join();
                    ThreadFive.join();
                    ThreadSix.join();
                    ThreadSeven.join();
                    ThreadEight.join();
                    ThreadNine.join();
                    ThreadTen.join();
                    ThreadEleven.join();
                    ThreadTwelve.join();
                    ThreadThirteen.join();
                    ThreadFourteen.join();
                    ThreadFifteen.join();
                    ThreadSixteen.join();
                    ThreadSeventeen.join();
                    ThreadEighteen.join();
                    ThreadNineteen.join();
                    ThreadTwenty.join();
                    System.out.println("转账关系导入结束"+df.format(new Date()));
                }catch (Exception e){
                    System.out.println(e);
                    hugeClient.close();
                }
         }
}